/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "data_log.h"

#ifdef SQL_LIBS
int sql_query_log(struct probe_descriptor probe[],struct configuration_struct *settings,char iso_time[],char data_unit[SQL_QUERY_LENGHT],float factor_abs,float factor_display) {
/* build and send an sql query to the sql server in order to log data fro ALL probes */
/* return EXIT_SUCCESS or EXIT_FAILURE */
/* we need to build a string like 'INSERT into table (field1,field2) values (value1,value2)' */
	float f_abs,f_display;
	int number,probe_index;

	sql_buf1[0]='\0';
	sql_buf2[0]='\0';
	for(probe_index=0;probe_index<settings->num_probes;probe_index++) {
		f_abs=factor_abs*probe[probe_index].sensitivity;
		f_display=factor_display*probe[probe_index].sensitivity;
		/* we write: time cpm equi_dose */
		fprintf(std_msg,"\t[%d] -> %4g [%s]\t%g [mR/h]\n",probe[probe_index].number,probe[probe_index].value,data_unit,probe[probe_index].value/f_display);
		if(settings->file_log==EXIT_SUCCESS) fprintf(probe[probe_index].log_file,"%s %4g %g\n",iso_time,probe[probe_index].value,probe[probe_index].value/f_abs);
		
		number=snprintf(sql_query,(size_t)(SQL_QUERY_LENGHT*sizeof(char)),"%s%d,",settings->sql_field,probe_index+1);
		if(number <= -1 || number >= (int)(SQL_QUERY_LENGHT*sizeof(char))) {
			perror("sql_query: query string building failure\n");
			return EXIT_FAILURE;
		}
		strncat(sql_buf1,sql_query,SQL_QUERY_LENGHT);
		number=snprintf(sql_query,(size_t)(SQL_QUERY_LENGHT*sizeof(char)),"%g,",probe[probe_index].value/f_abs);
		if(number <= -1 || number >= (int)(SQL_QUERY_LENGHT*sizeof(char))) {
			perror("sql_query: query string building failure\n");
			return EXIT_FAILURE;
		}
		strncat(sql_buf2,sql_query,SQL_QUERY_LENGHT);
	}
	number=snprintf(sql_query,(size_t)(SQL_QUERY_LENGHT*sizeof(char)),"INSERT into %s (%stimestamps) values (%s'%s')",settings->sql_table,sql_buf1,sql_buf2,iso_time);
	if(number <= -1 || number >= (int)(SQL_QUERY_LENGHT*sizeof(char))) {
		perror("sql_query: final query string building failure\n");
		return EXIT_FAILURE;
	}
	
	//fprintf(stderr,"Query=%s\n",sql_query);
	/* checking that the database is still there and if not, trying to reconnect */
	if(mysql_ping(&settings->mysql)){
		fprintf(std_err,"Trying to reconnect to the database...\n");
		mysql_close(&settings->mysql);
		if(sql_init(settings)== EXIT_FAILURE) {
			return EXIT_FAILURE;
		}
		return EXIT_FAILURE;
	}
	
	if(mysql_query(&settings->mysql,sql_query))
	{
		//fprintf(stderr,"User=\"%s\" Password=\"%s\"\n",settings->sql_user,settings->sql_password);
		fprintf(stderr,"Query \"%s\" failed for %s@%s (%s)\n",sql_query,settings->sql_user,settings->sql_host,mysql_error(&settings->mysql));
		return EXIT_FAILURE;
	}
	
	return EXIT_SUCCESS;
}
#endif

int data_log(struct probe_descriptor probe[],struct configuration_struct *settings) {
/* called by the timer (SIGALRM), write the cpm and dose for every probes */
/* return EXIT_SUCCESS or EXIT_FAILURE */
/* cpm_write calibration: 65 cps=1 mR/h of Co60 */
	struct tm *local_time;			/* local time structure */
	char iso_time[20];			/* string containing the ISO formated time */
	time_t timing;				/* time in seconds since EPOC */
	int probe_index;
	
	//fprintf(std_msg,"\tInto data_log\n");
	if(time(&timing) == ((time_t)-1)) {
		perror("data_log: Time reading failure - skipping event\n");
		return EXIT_FAILURE;
	}
	if((local_time=(struct tm*)localtime(&timing)) != NULL) { /* cast done because gcc was complaining. Why??? */
		strftime(iso_time,(size_t)20,"%Y-%m-%d-%H:%M:%S",localtime(&timing));
	} else {
		perror("data_log: Time conversion failure");
		return EXIT_FAILURE;
	}
	
	fprintf(std_msg,"%s\n",iso_time);
	if(settings->pulse_mode==EXIT_SUCCESS) {
	/* PULSE MODE */
		#ifdef SQL_LIBS
		if(settings->sql_log==EXIT_SUCCESS) {
			sql_query_log(probe,settings,iso_time,"raw",bicron_signal2Rh,bicron_signal2mRh);
		}
		else {
		#endif
			for(probe_index=0;probe_index<settings->num_probes;probe_index++) {
				/* we write: time cpm equi_dose */
				if(settings->file_log==EXIT_SUCCESS) fprintf(probe[probe_index].log_file,"%s %4g %g\n",iso_time,probe[probe_index].value,probe[probe_index].value/(bicron_signal2Rh*probe[probe_index].sensitivity));
				fprintf(std_msg,"\t[%d] -> %4g [raw]\t%g [mR/h]\n",probe[probe_index].number,probe[probe_index].value,probe[probe_index].value/(bicron_signal2mRh*probe[probe_index].sensitivity));
			}
		#ifdef SQL_LIBS
		}
		#endif
	}
	else {
	/* CPM MODE */
		#ifdef SQL_LIBS
		if(settings->sql_log==EXIT_SUCCESS) {
			sql_query_log(probe,settings,iso_time,"count",events2Rh,events2mRh);
		}
		else {
		#endif
			for(probe_index=0;probe_index<settings->num_probes;probe_index++) {
				/* we write: time cpm equi_dose */
				if(settings->file_log==EXIT_SUCCESS) fprintf(probe[probe_index].log_file,"%s %4g %g\n",iso_time,probe[probe_index].value*events2cpm,probe[probe_index].value/(events2Rh*probe[probe_index].sensitivity));
				fprintf(std_msg,"\t[%d] -> %4g [cpm]\t%g [mR/h]\n",probe[probe_index].number,probe[probe_index].value*events2cpm,probe[probe_index].value/(events2mRh*probe[probe_index].sensitivity));
				probe[probe_index].value=0;
			}
		#ifdef SQL_LIBS
		}
		#endif
	}
	flag_alrm=0;
	
	return EXIT_SUCCESS;
}
