/***************************************************************************
                          gm4lin.c  -  main program
                             -------------------
    begin                : Thu Dec 12 14:59:53 GST 2002
    copyright            : (C) 2003 by Mathias Bavay
    email                : mbavay@crtholdings.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "gm4lin.h"

/* global variables													 */
/*the declaration order is important, for data alignement issues */
FILE *std_msg;
FILE *std_err;

unsigned int flag_alrm;								/* flag for SIGALRM */
unsigned int flag_stop;								/* flag for the end of the recording */

#ifdef SQL_LIBS
char sql_buf1[SQL_QUERY_LENGHT],sql_buf2[SQL_QUERY_LENGHT],sql_query[SQL_QUERY_LENGHT];	/* in case of an SQL logging, we will need some buffers */
#endif

/* ******************************************************************************************* */

main(int argn,char *argv[]){
	struct timeval timeout;					/* time out structure for "select" */
	struct configuration_struct settings;			/* storage of everything linked to configuration of the software */
	struct probe_descriptor probe[SERIAL_PORTS_MAX];	/* array of probe descriptor pointers */
	fd_set file_descriptor_set;				/* file descriptor set of probes */
	float exponential_factor;				/* multiplication factor for pulse mode */
	int probe_index;					/* index of the probe */
	int events;						/* number of ports which have seen an event */
	int sensitivity[SERIAL_PORTS_MAX];			/* table of probes sensitivities */
	
	/* initializations */
	std_msg=stdout;
	std_err=stderr;
	flag_stop=0;
	if(settings_init(argn,argv,&settings,sensitivity)== EXIT_FAILURE) return EXIT_FAILURE;
	fprintf(std_msg,"Starting program...\npress CTRL+C to stop\n");
	/* probes initialization */
	fprintf(std_msg,"************\nInitializing probes...\n");
	if(probes_init(probe,&settings,sensitivity)== EXIT_FAILURE) return EXIT_FAILURE;

	#ifdef SQL_LIBS
	/* MySQL initialization if necessary */
	if(settings.sql_log==EXIT_SUCCESS) {
		if(sql_init(&settings)== EXIT_FAILURE) return EXIT_FAILURE;	
	}
	#endif
	
	/* launching sequencer (timer+signal catching) */
	if(sequencer_init(settings)== EXIT_FAILURE) return EXIT_FAILURE;
	
	/* Running probes in activity recording mode*/
	if (settings.pulse_mode==EXIT_FAILURE) {
		fprintf(std_msg,"************\nRunning probes in activity recording mode...(refresh delay=%d seconds)\n",settings.activity_data_write_sampling);
		do {
			/* re-initializing the set of file descriptors */
			FD_ZERO(&file_descriptor_set);
			/* building the set of file descriptors */
			for(probe_index=0;probe_index<settings.num_probes;probe_index++) {
				FD_SET(probe[probe_index].file_descriptor,&file_descriptor_set);
			}
			/* setting the time out */
			timeout.tv_sec=MAIN_LOOP_REFRESH_RATE;
			timeout.tv_usec=0;
			//fprintf(std_msg,"\tSelect waiting for input (fd_max+1=%d)...\n",file_descriptor_max+1);
			/* running probes */
			if(((events=select(settings.file_descriptor_max+1,&file_descriptor_set,NULL,NULL,&timeout)) == -1) && !flag_stop && !flag_alrm) {
				perror("main: select call failed");
				return EXIT_FAILURE;
			}
			if(events > 0) { /* when flag_alrm is ON, events <0 */
				//fprintf(std_msg,"\tEvent detected on %d port(s)\n",events);
				/* at least 1 port has seen something */
				for(probe_index=0;probe_index<settings.num_probes;probe_index++) {
					if(FD_ISSET(probe[probe_index].file_descriptor,&file_descriptor_set)) {
						/* one more event for that probe */
						probe[probe_index].value++;
						/* we need to read the port in order to clear it */
						clear_port(probe[probe_index].file_descriptor);
						}
				}	
			}
			if(flag_alrm && !flag_stop) {
				data_log(probe,&settings);
			}
		} while (!flag_stop);
	} 
	else {/* Running probes in pulse recording mode*/
		fprintf(std_msg,"************\nRunning probes in pulse recording mode...(refresh delay=%d seconds, decay time=%3.3f)\n",settings.pulse_data_write_sampling,INTEGRATION_RC);
		/* initializing the exponential factor which is going to multiply the signals */
		exponential_factor=(float)exp(-(double)settings.pulse_particule_sub_sampling/(double)INTEGRATION_RC*1.0e-3);
		/* reading data... */
		do {
			if(flag_alrm && !flag_stop) {
				flag_alrm=0;
				/* it is time to read data from the detectors */
				for(probe_index=0;probe_index<settings.num_probes;probe_index++) {
					probe[probe_index].value *= exponential_factor;
					if((probe_port(probe[probe_index].file_descriptor))>0) probe[probe_index].value += 1;
				}
				settings.pulse_mode_particules_sampling_count++;
				if(settings.pulse_mode_particules_sampling_count>=settings.pulse_mode_particules_sampling_write) {
					settings.pulse_mode_particules_sampling_count=0;
					data_log(probe,&settings);
				}
			}
		} while (!flag_stop);
	}
	
	
	/* stopping probes */
	fprintf(std_msg,"\tStopping probes...\n");
	for(probe_index=0;probe_index<settings.num_probes;probe_index++) {
		if ((stop_probe(&probe[probe_index])) == EXIT_FAILURE) {
			perror("main: probe can not be stopped");
			return EXIT_FAILURE;
		}
		/* closing the log file */
		if (settings.file_log==EXIT_SUCCESS) {
			if ((fclose(probe[probe_index].log_file)) != EXIT_SUCCESS){
				perror("main: Log file can not be closed");
				return EXIT_FAILURE;
			}
		}
	}
	#ifdef SQL_LIBS
	/* stopping MySQL if necessary */
	if(settings.sql_log=EXIT_SUCCESS) {
		mysql_close(&settings.mysql);
	}
	#endif
	/*****************************************************************************************************/
	/* ending the program */
	
	return EXIT_SUCCESS;
}
